function [V, Pg, f, lamda, success] =  OTopf(baseMVA, bus, gen, gencost, branch, ...
			Ybus, Yf, Yt, V0, Pg0, ref, pv, pq, alg, verbose)
%OTOPF  Solves an optimal power flow using constr from Optimization Tbx.
%   [V, Pg, f, lamda, success] =  OTopf(baseMVA, bus, gen, gencost, branch, ...
%           Ybus, Yf, Yt, V0, Pg0, ref, pv, pq, alg, verbose)
%   Assumes that Ybus, Yf, Yt, V, ref, pv, pq are consistent with
%   (or override) data in bus, gen, branch.

%   MATPOWER Version 1.0
%   by Ray Zimmerman, PSERC Cornell    9/16/97
%   Copyright (c) 1996, 1997 by Power System Engineering Research Center (PSERC)
%   See http://www.pserc.cornell.edu/ for more info.

%%----- initialization -----
gradient_check = 0;		%% flag for checking user supplied gradients
vVg = varVg(alg);

%% initialize arguments
if nargin < 15
	verbose = 0;		%% default: don't print stuff
end

%% initial state
V = V0;
Pg = Pg0;

%% constants
j = sqrt(-1);

%% define named indices into data matrices
[GEN_BUS, PG, QG, QMAX, QMIN, VG, MBASE, ...
	GEN_STATUS, PMAX, PMIN, MU_PMAX, MU_PMIN, MU_QMAX, MU_QMIN] = idx_gen;
[PW_LINEAR, POLYNOMIAL, MODEL, STARTUP, SHUTDOWN, N, COST] = idx_cost;

%%----- run opf -----
%% generator info
on = find(gen(:, GEN_STATUS));		%% which generators are on?

%% set up indexing for x
nb = size(bus, 1);
nl = size(branch, 1);
npv	= length(pv);
npq	= length(pq);
ng = npv + 1;
j1 = 1;			j2 = npv;			%% j1:j2 - V angle of pv buses
j3 = j2 + 1;	j4 = j2 + npq;		%% j3:j4 - V angle of pq buses
if vVg
	j5 = j4 + 1;	j6 = j4 + nb;	%% j5:j6 - V mag of all buses
else
	j5 = j4 + 1;	j6 = j4 + npq;	%% j5:j6 - V mag of pq buses
end
j7 = j6 + 1;	j8 = j6 + ng;		%% j7:j8 - P of generators
if CCV(alg)
	j9 = j8 + 1;	j10= j8 + ng;	%% j9:j10- c of generators
end

%% set up x
if vVg
	Vmag = abs(V);
else
	Vmag = abs(V(pq));
end
if CCV(alg)
	c = totcost(gencost, Pg * baseMVA);
else
	c = [];
end
x = [angle(V(pv)); angle(V(pq)); Vmag; Pg; c];

%% set up options for constr
opt = foptions;					%% get default options for optimization
opt(1) = verbose;				%% set verbose flag appropriately
opt(9) = gradient_check;		%% check user supplied gradients?
opt(13) = npv + 2 * npq;		%% number of equality constraints
opt(14) = 100;					%% maximum number of iterations

%% run constrained optimization
[fun, grad] = OTfungra(alg);
[x, opt, lamda] = constr(fun, x, opt, [], [], grad, ...
					baseMVA, bus, gen, gencost, branch, Ybus, Yf, Yt, V, ref, pv, pq, alg);

%% get final objective function value & constraint values
[f, g] = feval(fun, x, baseMVA, bus, gen, gencost, branch, Ybus, Yf, Yt, V, ref, pv, pq, alg);

%% check for convergence
if opt(10) >= opt(14) | max(abs(g(1:opt(13)))) > opt(4) | max(g((opt(13)+1):length(g))) > opt(4)
	success = 0;			%% did NOT converge
else
	success = 1;			%% DID converge
end

%% reconstruct V
if vVg
	Va = zeros(nb, 1);
	Va(ref) = angle(V(ref));
	Vm = x(j5:j6);
else
	Va = angle(V);	Vm = abs(V);
	Vm(pq) = x(j5:j6);
end
Va(pv) = x(j1:j2);
Va(pq) = x(j3:j4);
V = Vm .* exp(j * Va);

%% grab Pg
Pg = x(j7:j8);					%% power generation in p.u.

return;

% foptions stuff for constr
%
% OPTIONS(1)-Display parameter (Default:0). 1 displays some results
% OPTIONS(2)-Termination tolerance for X.(Default: 1e-4).
% OPTIONS(3)-Termination tolerance on F.(Default: 1e-4).
% OPTIONS(4)-Termination criterion on constraint violation.(Default: 1e-6)
% OPTIONS(5)-Algorithm: Strategy:	 Not always used.
% OPTIONS(6)-Algorithm: Optimizer: Not always used.
% OPTIONS(7)-Algorithm: Line Search Algorithm. (Default 0)
% OPTIONS(8)-Function value. (Lambda in goal attainment. )
% OPTIONS(9)-Set to 1 if you want to check user-supplied gradients
% OPTIONS(10)-Number of Function and Constraint Evaluations.
% OPTIONS(11)-Number of Function Gradient Evaluations.
% OPTIONS(12)-Number of Constraint Evaluations
% OPTIONS(13)-Number of equality constraints.
% OPTIONS(14)-Maximum number of iterations. (Default 100*no. of variables)
% OPTIONS(15)-Used in goal attainment for special objectives.
% OPTIONS(16)-Minimum change in variables for finite difference gradients.
% OPTIONS(17)-Maximum change in variables for finite difference gradients.
% OPTIONS(18)- Step length. (Default 1 or less).
% [x, OPTIONS,lambda, HESS]=constr(FUN,x,OPTIONS,VLB,VUB,GRADFUN,P1,P2,P3,P4,P5,P6,P7,P8,P9,P10,P11,P12,P13,P14,P15);
